@@ -3,7 +3,7 @@
 #' This package provides a framework for investigating large scale set data with
 #' the use of hierarchical clustering. While hierarchical clustering has been
 #' employed on set data numerous times, by converting the presence/absence
-#' matrix to a distance matrix and using \code{\link[stats]{hclust}}, this
+#' matrix to a distance matrix and using [stats::hclust()], this
 #' approach completely removes any notion of underlying set structure from the
 #' data. hierarchicalSets instead performs a clustering directly using set
 #' algebra by continuously merging sets with the largest intersection (for ties
@@ -12,8 +12,8 @@
 #' intersectionStack plot is a scalable pendant to Venn diagrams (showing the
 #' same information but using a different visual mapping).
 #'
-#' @seealso \code{\link{create_hierarchy}} For constructing HierarchicalSet
-#' object and \code{\link{plot.HierarchicalSet}} for visualization apporaches.
+#' @seealso [create_hierarchy()] For constructing HierarchicalSet
+#' object and [plot.HierarchicalSet()] for visualization apporaches.
 #'
 #' @docType package
 #'
@@ -24,3 +24,9 @@
 #' @import ggplot2
 #'
 NULL
+
+globalVariables(c(
+    "x",
+    "y",
+    "group"
+))
